%{
#include <stdlib.h>
#include <limits.h>
#include <inttypes.h>
#include <string.h>
#include <errno.h>

#include <net/if.h>
#include <netinet/in.h>
#include <net/pfvar.h>

#include "parse.h"
#include "parser.h"

%}

%%

state		{ return STATE;}
on			{ return ON;}
out			{ return OUT;}
in			{ return IN;}
proto   	{ return PROTO;}
from		{ return FROM;}
to			{ return TO;}
using   	{ return USING;}
id			{ return ID;}
cid			{ return CID;}
expire  	{ return EXPIRE;}
timeout     { return TIMEOUT;}
src			{ return SRC;}
dst			{ return DST;}
seq			{ return SEQ;}
max_win 	{ return MAX_WIN;}
wscale		{ return WSCALE;}
mss			{ return MSS;}
no-scrub	{ return NOSCRUB;}
scrub		{ return SCRUB;}
flags		{ return FLAGS;}
ttl			{ return TTL;}
mode		{ return MODE;}
[0-9]+		{ char *ep;
			  errno = 0;
			  yylval.num = strtoumax(yytext, &ep, 10);
			  if (errno == ERANGE && yylval.num == UINTMAX_MAX)
					yyfatal("Number out of range");
			  return NUMBER;
			}

[A-Za-z0-9:\[][A-Za-z0-9\[\]_:%\.-]* { yylval.str = strdup(yytext);
							    if (yylval.str == NULL)
									yyfatal("Not enough memory");
								return STRING;
							  }


\n			{ lineno ++; }

%%


void
yyfatal(const char *s)
{
	yyerror(s);
	exit(EXIT_FAILURE);
}

void
yyerror(const char *s)
{
	printf("line %d: %s at [%s]\n", lineno, s, yytext);
}


int
parse(FILE *fp, struct pfioc_states* s)
{
	yyin = fp;

	lineno = 1;

	states = s;
	allocated = 0;
	memset(s, 0, sizeof(*s));

	if (yyparse()) {
		printf("parse failed, line %d.\n", lineno);
		return(-1);
	}

	return(0);
}
